@@ -15,5 +15,13 @@ from clio._clio import (
     ClioError, PublishError, EditError, UpdateError, DeleteError, RevertError)
 
 from clio._clio import Model, Table
-from clio._clio import workflow_properties
+from clio._clio import workflow_properties, history
+from clio._clio import (
+    HISTORY_CREATE,
+    HISTORY_PUBLISH,
+    HISTORY_EDIT,
+    HISTORY_CHANGE,
+    HISTORY_DELETE,
+    HISTORY_EDIT_DELETE,
+    HISTORY_ARCHIVE)
 from clio.sequence import Sequence
