@@ -63,14 +63,19 @@ def Table(name, metadata, *args, **kw):
     return SATable(name, metadata,
                    Column('id', Integer, primary_key=True),
                    Column('code', Integer, nullable=False, index=True),
-                   Column('status', Integer, nullable=False),
+                   Column('status', Integer, nullable=False, index=True),
                    Column('created_timestamp', DateTime, nullable=False),
                    Column('published_start_timestamp', DateTime, nullable=True,
                           index=True),
                    Column('published_end_timestamp', DateTime, nullable=True,
                           index=True),
-                   Column('published_userid', Unicode(64), nullable=True),
-                   Column('changed_userid', Unicode(64), nullable=True),
+                   Column('changed_timestamp', DateTime, nullable=True),
+                   Column('created_userid', Unicode(64), nullable=True,
+                          index=True),
+                   Column('published_userid', Unicode(64), nullable=True,
+                          index=True),
+                   Column('changed_userid', Unicode(64), nullable=True,
+                          index=True),
                    UniqueConstraint('code', 'status', name='workflow'),
                    *args, **kw)
 
@@ -148,9 +153,15 @@ class Model(object):
         self.code = code
         self.status = NEW
         self.created_timestamp = self._now()
+        self.created_userid = self._get_userid()
+
+        # when you create a new object from scratch, you've
+        # also changed it
+        self.changed_timestamp = self.created_timestamp
+        self.changed_userid = self.created_userid
+
         self.published_start_timestamp = None
         self.published_end_timestamp = None
-        self.edited_userid = None
         self.published_userid = None
         
     def publish(self):
@@ -356,6 +367,14 @@ class Model(object):
     def is_deleted(self):
         return self.status in DELETED_STATUSES
 
+    def editable_version(self):
+        """Give version that's visible in to an edit UI, if any.
+
+        If only archived versions exist, None is returned.
+        """
+        q = self._versions_query()
+        return q.filter(self.__class__.status.in_(EDITABLE_STATUSES)).first()
+
     def compare_relation(self, relation_name):
         if self.status == EDITABLE:
             editable = self
@@ -383,6 +402,7 @@ class Model(object):
         return ComparisonResult(editable, deleted, unchanged)        
 
     def mark_changed(self):
+        self.changed_timestamp = self._now()
         self.changed_userid = self._get_userid()
             
     # PRIVATE
@@ -441,8 +461,7 @@ class Model(object):
 
     def _get_next_archive_status(self):
         session = object_session(self)
-        cls = self.__class__
-        status = session.query(func.max(cls.status)).first()[0]
+        status = session.query(func.max(self.__class__.status)).first()[0]
         if status < ARCHIVED:
             status = ARCHIVED
         else:
@@ -456,13 +475,15 @@ class Model(object):
         for prop in self._get_other_props():
             value = getattr(self, prop.key)
             setattr(result, prop.key, value)
+        # the copy is not published
         result.published_start_timestamp = None
         result.published_end_timestamp = None
+        result.published_userid = None
+        # the copy has just been created
         result.created_timestamp = result._now()
+        result.created_userid = self._get_userid()
         del result.id
         result.status = UPDATABLE
-        result.changed_userid = None
-        result.published_userid = None
         return result
 
     def _create_update_copy_onto(self, target):
@@ -476,11 +497,13 @@ class Model(object):
                 value = getattr(self, prop.key)
                 setattr(result, prop.key, value)
         result.status = UPDATABLE
+        # the copy is not published
         result.published_start_timestamp = None
         result.published_end_timestamp = None
-        result.created_timestamp = result._now()
-        result.changed_userid = None
         result.published_userid = None
+        # the copy has just been created
+        result.created_timestamp = result._now()
+        result.created_userid = self._get_userid()
         return result
 
     def _create_edit_copy(self):
@@ -555,16 +578,179 @@ class Model(object):
                 orig_values.remove(related)
                 values.append(related)
 
-    def _version(self, status):
+    def _versions_query(self):
         session = object_session(self)
-        cls = self.__class__
-        return session.query(cls).filter(and_(
-                cls.code == self.code,
-                cls.status == status)).first()
+        return session.query(self.__class__).filter(
+            self.__class__.code == self.code)
+        
+    def _version(self, status):
+        return self._versions_query().filter(
+            self.__class__.status == status).first()
 
+    def _has_older_archived_version(self):
+        q = self._versions_query()
+        if self.is_published():
+            return q.filter(
+                self.__class__.status >= ARCHIVED).first() is not None
+        else:
+            return q.filter(
+                and_(self.__class__.status >= ARCHIVED,
+                     self.__class__.status < self.status)).first() is not None
+        
+    def _is_last_version(self):
+        q = self._versions_query()
+        # if there is a newer archived version, this isn't the last version
+        if q.filter(
+            and_(self.__class__.status >= ARCHIVED,
+                 self.__class__.status > self.status)).first() is not None:
+            return False
+        # if there is a non-archived version, this isn't the last version
+        return q.filter(and_(self.__class__.status < ARCHIVED)).first() is None
+        
     def _deleted_version(self):
         for status in DELETED_STATUSES:
             version = self._version(status)
             if version is not None:
                 return version
         return None
+
+
+HISTORY_CREATE = u'create'
+HISTORY_PUBLISH = u'publish'
+HISTORY_EDIT = u'edit'
+HISTORY_CHANGE = u'change'
+HISTORY_DELETE = u'delete'
+HISTORY_EDIT_DELETE = u'edit_delete'
+HISTORY_ARCHIVE = u'archive'
+
+class HistoryEvent(object):
+    def __init__(self, event_type, timestamp, version, userid):
+        """Construct a HistoryEvent.
+
+        event_type: a historical event (HISTORY_CREATE, etc).
+        timestamp: the timestamp of the historical event.
+        version: the version that was involved in this event.
+        userid: the userid who initiated the event. Can be None
+                if this cannot be reconstructed.
+        """
+        self.event_type = event_type
+        self.timestamp = timestamp
+        self.version = version
+        self.userid = userid
+
+
+def _history_key(event):
+    # archiving should always take place before publishing if the time is
+    # the same, guarantee consistent sorting order
+    # similarly, editing should always take place before change if time
+    # is the same
+    if event.event_type == HISTORY_PUBLISH:
+        extra_sort = 1
+    elif event.event_type == HISTORY_ARCHIVE:
+        extra_sort = 0
+    elif event.event_type == HISTORY_CHANGE:
+        extra_sort = 1
+    elif event.event_type == HISTORY_EDIT:
+        extra_sort = 0
+    else:
+        extra_sort = 0
+    # microseconds aren't recorded by the database
+    timestamp = event.timestamp.replace(microsecond=0)
+    return timestamp, extra_sort
+
+def _add_changed_event(result, version):
+    # in updated databases it is possible for changed_timestamp to
+    # be None
+    if version.changed_timestamp is None:
+        return
+    if version.changed_timestamp > version.created_timestamp:
+        result.append(HistoryEvent(HISTORY_CHANGE,
+                                   version.changed_timestamp,
+                                   version,
+                                   version.changed_userid))
+    
+def history(versions):
+    """Return historical information about ORM-mapped objects.
+
+    Historical information consists of a list of historical events.
+    This includes the creation of objects and their publication and
+    archiving, and the version of the object involved. These events are 
+    sorted in chronological order.
+
+    The input to this function is an iterable of versions. These should
+    include all historical versions of all objects that we want
+    chronological information about. Usually such a list of versions
+    can be obtained by using a relation or by doing a query.
+    """
+    result = []
+    for version in versions:
+        if version.status == NEW:
+            result.append(HistoryEvent(HISTORY_CREATE,
+                                       version.created_timestamp,
+                                       version,
+                                       version.created_userid))
+            _add_changed_event(result, version)
+        elif version.status == EDITABLE:
+            result.append(HistoryEvent(HISTORY_EDIT,
+                                       version.created_timestamp,
+                                       version,
+                                       version.created_userid))
+            _add_changed_event(result, version)
+        elif version.status == UPDATABLE:
+            result.append(HistoryEvent(HISTORY_EDIT,
+                                       version.created_timestamp,
+                                       version,
+                                       version.created_userid))
+            _add_changed_event(result, version)
+        elif version.status in DELETED_STATUSES:
+            result.append(HistoryEvent(HISTORY_EDIT_DELETE,
+                                       version.created_timestamp,
+                                       version,
+                                       version.created_userid))
+            _add_changed_event(result, version)
+        elif version.status in PUBLISHED_STATUSES:
+            if version._has_older_archived_version():
+                result.append(HistoryEvent(HISTORY_EDIT,
+                                           version.created_timestamp,
+                                           version,
+                                           version.created_userid))
+            else:
+                result.append(HistoryEvent(HISTORY_CREATE,
+                                           version.created_timestamp,
+                                           version,
+                                           version.created_userid))
+            _add_changed_event(result, version)
+            result.append(HistoryEvent(HISTORY_PUBLISH,
+                                       version.published_start_timestamp,
+                                       version,
+                                       version.published_userid))
+        elif version.status >= ARCHIVED:
+            if version._has_older_archived_version():
+                result.append(HistoryEvent(HISTORY_EDIT,
+                                           version.created_timestamp,
+                                           version,
+                                           version.changed_userid))
+            else:
+                result.append(HistoryEvent(HISTORY_CREATE,
+                                           version.created_timestamp,
+                                           version,
+                                           version.changed_userid))
+            _add_changed_event(result, version)
+            result.append(HistoryEvent(HISTORY_PUBLISH,
+                                       version.published_start_timestamp,
+                                       version,
+                                       version.published_userid))
+            if version._is_last_version():
+                result.append(HistoryEvent(HISTORY_DELETE,
+                                           version.published_end_timestamp,
+                                           version,
+                                           version.published_userid))
+            else:
+                result.append(HistoryEvent(HISTORY_ARCHIVE,
+                                           version.published_end_timestamp,
+                                           version,
+                                           None))
+
+    result.sort(key=_history_key)
+    return result
+
