@@ -32,3 +32,11 @@ class Sequence(object):
     def reinitialize(self):
         self._initialized = False
 
+    def update(self, session, nr):
+        """Update from table.
+        """
+        if not self._initialized:
+            self._initialize(session)
+        u = self.table.update().values(id=nr)
+        session.execute(u)
+        
