@@ -232,9 +232,9 @@ class ClioTestCase(unittest.TestCase):
     def test_a_mark_changed(self):
         one = A(a_code(self.session), u'One')
         self.session.add(one)
-        self.assertEquals(None, one.changed_userid)
-        one.mark_changed()
         self.assertEquals(u'userid_0', one.changed_userid)
+        one.mark_changed()
+        self.assertEquals(u'userid_1', one.changed_userid)
         
     def test_a_publish(self):
         """We can publish a new A.
@@ -268,10 +268,10 @@ class ClioTestCase(unittest.TestCase):
         one = A(a_code(self.session), u'One')
         self.session.add(one)
         self.assertEquals(None, one.published_userid)
-        
+        self.assertEquals(u'userid_0', one.changed_userid)
         one = one.publish()
 
-        self.assertEquals(u'userid_0', one.published_userid)
+        self.assertEquals(u'userid_1', one.published_userid)
     
     def test_cannot_change_published(self):
         pass # to be implemented
@@ -480,17 +480,28 @@ class ClioTestCase(unittest.TestCase):
     def test_a_archive_userid(self):
         """When we publish an edited A, the previous version is archived.
         """
-        one1 = self._get_published()
+        one1 = A(a_code(self.session), u'One')
+        self.session.add(one1)
+        one1 = one1.publish()
+        self.session.commit()
         one_editable2 = one1.edit()
         one_editable2.message = u'One 2'
+        one_editable2.mark_changed()
         one2 = one_editable2.publish()
         one_editable3 = one2.edit()
         one_editable3.message = u'One 3'
+        one_editable3.mark_changed()
         one3 = one_editable3.publish()
 
-        self.assertEquals(u'userid_0', one1.published_userid)
-        self.assertEquals(u'userid_1', one2.published_userid)
-        self.assertEquals(u'userid_2', one3.published_userid)
+        self.assertEquals(u'userid_0', one1.created_userid)
+        self.assertEquals(u'userid_0', one1.changed_userid)
+        self.assertEquals(u'userid_1', one1.published_userid)
+        self.assertEquals(u'userid_2', one2.created_userid)
+        self.assertEquals(u'userid_3', one2.changed_userid)
+        self.assertEquals(u'userid_4', one2.published_userid)
+        self.assertEquals(u'userid_5', one3.created_userid)
+        self.assertEquals(u'userid_6', one3.changed_userid)
+        self.assertEquals(u'userid_7', one3.published_userid)
     
     def test_a_archive_updatable(self):
         """When we publish an updated, the previous version is archived.
@@ -1251,7 +1262,162 @@ class ClioTestCase(unittest.TestCase):
         self.assertEquals([b1_deleted], compared.deleted)
         self.assertEquals([], compared.edited)
         self.assertEquals([one.bs[1]], compared.unchanged)
+
+    def test_changed_userid_new(self):
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        self.assertEquals(one.created_userid, one.changed_userid)
+
+    def test_changed_userid_edit(self):
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        one = one.publish()
+        self.session.commit()
+        one_editable = one.edit()
+        self.assertEquals('userid_0', one.created_userid)
+        self.assertEquals('userid_0', one.changed_userid)
+        self.assertEquals('userid_1', one.published_userid)
+        self.assertEquals('userid_2', one_editable.created_userid)
+        self.assertEquals('userid_0', one_editable.changed_userid)
+
+    def test_history1(self):
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        one = one.publish()
+        self.session.commit()
+        one_editable = one.edit()
+        one = one.publish()
+        one_editable = one.edit()
+        one = one.publish()
+
+        versions = self.session.query(A).filter(A.code == one.code)
         
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE,
+                           clio.HISTORY_PUBLISH,
+                           clio.HISTORY_EDIT,
+                           clio.HISTORY_ARCHIVE,
+                           clio.HISTORY_PUBLISH,
+                           clio.HISTORY_EDIT,
+                           clio.HISTORY_ARCHIVE,
+                           clio.HISTORY_PUBLISH],
+                          event_types)
+        
+    def test_history_just_create(self):
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        self.session.commit()
+
+        versions = self.session.query(A).filter(A.code == one.code)
+        
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE],
+                          event_types)
+
+    def test_history_create_and_change(self):
+        one = A(a_code(self.session), u'One')
+        one.mark_changed()
+        self.session.add(one)
+        self.session.commit()
+
+        versions = self.session.query(A).filter(A.code == one.code)
+        
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE, clio.HISTORY_CHANGE],
+                          event_types)
+
+    def test_history_create_and_change_and_publish(self):
+        one = A(a_code(self.session), u'One')
+        one.mark_changed()
+        one.publish()
+        self.session.add(one)
+        self.session.commit()
+        
+        versions = self.session.query(A).filter(A.code == one.code)
+        
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE, clio.HISTORY_CHANGE,
+                           clio.HISTORY_PUBLISH],
+                          event_types)
+
+    def test_history_create_and_change_and_publish_and_edit(self):
+        one = A(a_code(self.session), u'One')
+        one.mark_changed()
+        one.publish()
+        self.session.add(one)
+        self.session.commit()
+        one_editable = one.edit()
+        self.session.commit()
+        
+        versions = self.session.query(A).filter(A.code == one.code)
+
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE, clio.HISTORY_CHANGE,
+                           clio.HISTORY_PUBLISH, clio.HISTORY_EDIT],
+                          event_types)
+
+    def test_history_create_and_change_and_publish_and_edit_and_change(self):
+        one = A(a_code(self.session), u'One')
+        one.mark_changed()
+        one.publish()
+        self.session.add(one)
+        one_editable = one.edit()
+        one_editable.mark_changed()
+        self.session.commit()
+                                  
+        versions = self.session.query(A).filter(A.code == one.code)
+
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE, clio.HISTORY_CHANGE,
+                           clio.HISTORY_PUBLISH, clio.HISTORY_EDIT,
+                           clio.HISTORY_CHANGE],
+                          event_types)
+
+    def test_history_create_and_change_and_publish_and_delete(self):
+        one = A(a_code(self.session), u'One')
+        one.mark_changed()
+        one.publish()
+        self.session.add(one)
+        one_deletable = one.delete()
+        one_deletable.publish()
+        self.session.commit()
+                                  
+        versions = self.session.query(A).filter(A.code == one.code)
+
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE, clio.HISTORY_CHANGE,
+                           clio.HISTORY_PUBLISH, clio.HISTORY_DELETE],
+                          event_types)
+
+    def test_history_create_and_publish_and_submit_for_deletion(self):
+        one = A(a_code(self.session), u'One')
+        one.publish()
+        self.session.add(one)
+        one_deletable = one.delete()
+        self.session.commit()
+                                  
+        versions = self.session.query(A).filter(A.code == one.code)
+
+        events = clio.history(versions)
+        
+        event_types = [event.event_type for event in events]
+        self.assertEquals([clio.HISTORY_CREATE,
+                           clio.HISTORY_PUBLISH, clio.HISTORY_EDIT_DELETE],
+                          event_types)
 
 def count_status(l, status):
     result = 0
